/*
 * tai8570.c
 *
 * Version 1.02
 *
 * Simon Melhuish
 * http://melhuish.info/simon/
 * http://oww.sourceforge.net/
 *
 * Derived from source by AAG
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "ownet.h"
#include "swt12.h"
#include "findtype.h"
#include "owfile.h"

// Constant definition
#define SWITCH_FAMILY      0x12
#define MAXDEVICES         15

int reader = 0, writer = 0;
uchar SwitchSN[MAXDEVICES+1][8];	//the serial numbers for the devices

// local
int getNumber (int min, int max);

/**
 *
 * @author  Aitor
 */

// Updated by Simon Melhuish, with ref. to AAG C++ code

#define SEC_READW4     "111011010000"
#define SEC_READW2     "111010110000"
#define SEC_READW1     "111010101000"
#define SEC_READW3     "111011001000"

#define SEC_READD1     "11110100000"
#define SEC_READD2     "11110010000"
#define SEC_RESET      "101010101010101000000"

typedef unsigned char byte;
typedef int bool;
#define false 0
#define true  1

static char SW1[20], SW2[20], SW3[20], SW4[20];

//static  byte CHANNEL_ACCESS =(byte)0xF5 ; 
static byte CFG_READ = (byte) 0xEC;	// '11101100'   Configuracin de lectura para DS2407
static byte CFG_WRITE = (byte) 0x8C;	// '10001100'  Configuracin de Escritura para DS2407
static byte CFG_READPULSE = (byte) 0xC8;	// '11001000'  Configuracin de lectura de Pulso de conversion para DS2407



static int C1 = 0, C2 = 0, C3 = 0, C4 = 0, C5 = 0, C6 = 0;

static int D1 = 0, D2 = 0;

static bool validCFG = false;

//static double height_meters=0.0;

static double Pressure_mbar = 0.0;

static double Temp_C = 0.0;


static void SendPressureBit (int portnum, bool i);
static bool CheckConversion (int portnum);

//----------------------------------------------------------------------
//      SUBROUTINE - WriteSwitch12 - Adapted from SetSwitch12 Dallas code
//
//  This routine sets the channel state of the specified DS2406
//
// 'portnum'     - number 0 to MAX_PORTNUM-1.  This number was provided to
//                 OpenCOM to indicate the port number.
// 'SerialNum'   - Serial Number of DS2406 to set the switch state
// 'st'          - Config byte to write
//
//  Returns: TRUE(1)  State of DS2406 set and verified
//           FALSE(0) could not set the DS2406, perhaps device is not
//                    in contact
//
int
WriteSwitch12 (int portnum, uchar * SerialNum, ushort st)
{
	//ushort st;
	int rt = FALSE;
	uchar send_block[30];
	int send_cnt = 0;

	setcrc16 (portnum, 0);

	// set the device serial number to the counter device
	owSerialNum (portnum, SerialNum, FALSE);

	// access the device
	if (owAccess (portnum))
	{
		// create a block to send that reads the counter
		// write status command
		send_block[send_cnt++] = 0x55;
		docrc16 (portnum, 0x55);

		// address of switch state
		send_block[send_cnt++] = 0x07;
		docrc16 (portnum, 0x07);
		send_block[send_cnt++] = 0x00;
		docrc16 (portnum, 0x00);

		send_block[send_cnt++] = (uchar) st;
		docrc16 (portnum, st);

		// read CRC16
		send_block[send_cnt++] = 0xFF;
		send_block[send_cnt++] = 0xFF;

		// now send the block
		if (owBlock (portnum, FALSE, send_block, send_cnt))
		{
			// perform the CRC16 on the last 2 bytes of packet
			docrc16 (portnum, send_block[send_cnt - 2]);

			// verify crc16 is correct
			if (docrc16 (portnum, send_block[send_cnt - 1]) ==
			    0xB001)
				rt = TRUE;
		}
	}

	// return the result flag rt
	return rt;
}

// Open (i.e. o/p goes high) the channel A switch
static int
OpenA (int portnum, uchar * SerialNum)
{
	byte I = 0XFF, res = 0;

	int Ind = 5;
	do
	{
		owSerialNum (portnum, SerialNum, FALSE);
		I = ReadSwitch12 (portnum, false);
		//res = I | 0X20;
		WriteSwitch12 (portnum, SerialNum, I | 0X20);
		I = ReadSwitch12 (portnum, false);
		Ind--;
		res = (I & 0XDF);
		if (res != 0)
			return true;
	}
	while (Ind > 0);	//&& ((I & 0XDF)!=0));
	//while ((Ind>0) && ((I & 0XDF)!=0));
	return (Ind > 0);
}

// Open (i.e. o/p goes high) the channel B switch
static int
OpenB (int portnum, uchar * SerialNum)
{
	byte I = 0XFF, res = 0;

	int Ind = 5;
	do
	{
		owSerialNum (portnum, SerialNum, FALSE);
		I = ReadSwitch12 (portnum, false);
		//res = I | 0X40;
		WriteSwitch12 (portnum, SerialNum, I | 0X40);
		I = ReadSwitch12 (portnum, false);
		Ind--;
		res = (I & 0XBF);
		if (res != 0)
			return true;
	}
	while (Ind > 0);	//&& ((I & 0XDF)!=0));
	//while ((Ind>0) && ((I & 0XDF)!=0));
	return (Ind > 0);
}

// Open B (data in/out) on both reader and writer
static bool
OpenPIOS_B (int portnum)
{
	if (!OpenB (portnum, SwitchSN[reader]))
		return FALSE;
	if (!OpenB (portnum, SwitchSN[writer]))
		return FALSE;

	return TRUE;
}

// Open A (clock) on both reader and writer
static bool
OpenPIOS_A (int portnum)
{
	if (!OpenA (portnum, SwitchSN[reader]))
		return FALSE;
	if (!OpenA (portnum, SwitchSN[writer]))
		return FALSE;

	return TRUE;
}

// Send a Channel Access command sequence
static int
ChannelAccess (int portnum, uchar * SerialNum, byte AChanC1, byte AChanC2,
	       byte * ACInfo)
{
	owSerialNum (portnum, SerialNum, FALSE);
	if (!owAccess (portnum))
	{
		fprintf (stderr, "owAccess failed in ChannelAccess\n");
		return false;
	}
	owTouchByte (portnum, 0XF5);
	owTouchByte (portnum, AChanC1);
	owTouchByte (portnum, AChanC2);
	*ACInfo = owTouchByte (portnum, 0XFF);

	return true;
}

//---------------------------------------------------------------------------
static int
ConfigWrite (int portnum)
{
	byte ACInfo = 0;
	return ChannelAccess (portnum, SwitchSN[writer], CFG_WRITE, 0xFF,
			      &ACInfo);

}

//---------------------------------------------------------------------------
static int
ConfigRead (int portnum)
{
	byte ACInfo = 0;
	return ChannelAccess (portnum, SwitchSN[reader], CFG_READ, 0xFF,
			      &ACInfo);

}

//---------------------------------------------------------------------------
static int
ConfigReadPulse (int portnum)
{
	byte ACInfo = 0;
	return ChannelAccess (portnum, SwitchSN[reader], CFG_READPULSE, 0xFF,
			      &ACInfo);

}



static bool
WritePressureSensor (int portnum, char *Cmd)
{
	bool rslt = false;
	int lng, i;
	//char command[64] = "";
	//printf ("WritePressureSensor(%d, \"%s\")\n", portnum, Cmd);
	//strcpy (command, Cmd);
	//strcat (command, "0");
	//sprintf(command, "%s0", Cmd) ;
	//Cmd=Cmd+"0";
	lng = strlen (Cmd);
	if (ConfigWrite (portnum))
	{
		for (i = 0; i < lng; i++)
		{
			if (Cmd[i] == '0')
				SendPressureBit (portnum, false);
			else
				SendPressureBit (portnum, true);
		}
    SendPressureBit (portnum, false);
		rslt = true;
	}
	//printf ("-> %d\n", rslt);
	return rslt;
}

static void
SendPressureBit (int portnum, bool i)
{
	//try {
	if (i)
		owTouchByte (portnum, 0x0e);
	else
		owTouchByte (portnum, 0x04);

	/*{
	 * owTouchBit(portnum, false);
	 * owTouchBit(portnum, true);
	 * owTouchBit(portnum, true);
	 * owTouchBit(portnum, true);
	 * owTouchBit(portnum, false);
	 * owTouchBit(portnum, false);
	 * }else{
	 * owTouchBit(portnum, false);
	 * owTouchBit(portnum, false); 
	 * owTouchBit(portnum, true);
	 * owTouchBit(portnum, false);
	 * owTouchBit(portnum, false);
	 * owTouchBit(portnum, false);
	 * } */
	//} catch (Exception e){}
}

  /******************************************************/

static bool
ReadPressureBit (int portnum)
{
	int res = 0;
	res = owReadByte (portnum);
	//printf("0x%x\n", res) ;
	res &= 0x80;
	owWriteByte (portnum, 0xfa);

	return (res != 0);

#if 0
	bool rslt = false;
	bool t;
	//try {
	t = owTouchBit (portnum, 1);	// Read PIO A #1
	t = owTouchBit (portnum, 1);	// Read PIO B #1
	t = owTouchBit (portnum, 1);	// Read PIO A #2
	t = owTouchBit (portnum, 1);	// Read PIO B #2
	t = owTouchBit (portnum, 1);	// Read PIO A #3
	t = owTouchBit (portnum, 1);	// Read Pio B #3
	t = owTouchBit (portnum, 1);	// Read Pio A #4
	rslt = owTouchBit (portnum, 1);	// Read Pio B #4

	owTouchBit (portnum, false);	//  Write PIO A #1
	owTouchBit (portnum, true);	// Write PIO B #1
	owTouchBit (portnum, false);	//  Write PIO A #2
	owTouchBit (portnum, true);	// Write PIO B #2
	owTouchBit (portnum, true);	// Write PIO A #3
	owTouchBit (portnum, true);	// Write PIO B #3
	owTouchBit (portnum, true);	// Write PIO A #4
	owTouchBit (portnum, true);	// Write PIO B #4

	//}catch (Exception e){
	//    System.out.println(e);
	//}
	//printf ("0x%x ", rslt);
	return rslt;
#endif
}

/******************************************************/
static char *
ReadPressureSensor (int portnum, byte CountBit)
{
	static char StringBuffer[64];
	int i;

	strcpy (StringBuffer, "");
	if (ConfigRead (portnum))
	{
		for (i = 0; i < CountBit; i++)
		{
			if (ReadPressureBit (portnum))
				strcat (StringBuffer, "1");
			else
				strcat (StringBuffer, "0");
		}
		//StringBuffer[i] = '\0' ;
	}
	else
	{
		fprintf (stderr, "ConfigRead failed\n");
	}
	return StringBuffer;
}

/******************************************************/
static bool
PressureReset (int portnum)
{
	bool rsl = false;
	if (!OpenPIOS_A (portnum))
		return false;	// Inspired by AAG C++ version
	if (!ConfigWrite (portnum))
		return false;	// Inspired by AAG C++ version
	rsl = WritePressureSensor (portnum, SEC_RESET);
	return rsl;
}

/******************************************************/
static char *
ReadSensorValueStr (int portnum, char *cmd, char *Lect)
{
	/*if (! InitBothPIOA() )return false;
	 * if (! WritePressureSensor(Comm))return false;
	 * if (!InitBothPIOA()) return false;
	 * ReadPressureSensor(16,Lect) ;
	 * if (! InitBothPIOB ())return false;
	 * return  true; */

	//static char Lect[64] = "" ;
	Lect[0] = '\0';
	//printf ("ReadSensorValueStr(%d, \"%s\")\n", portnum, cmd);
	if (!OpenPIOS_A (portnum))
		return FALSE;
	if (WritePressureSensor (portnum, cmd))
	{
		bool t;
		if (!OpenPIOS_A (portnum))
			return FALSE;
		strcpy (Lect, ReadPressureSensor (portnum, (byte) 16));
		t = OpenPIOS_B (portnum);
	}
	else
	{
		fprintf (stderr, "WritePressureSensor failed\n");
	}
	//printf ("Lect-> \"%s\"\n", Lect);
	return Lect;
}

/******************************************************/
static int
ReadSensorValue (int portnum, char *Comm)
{
	char Lect[30];

	strcpy (Lect, "");
	int Res = 0;
	int i = 0;

	if (!PressureReset (portnum))
		return 0;
	if (!WritePressureSensor (portnum, Comm))
		return 0;
	if (!CheckConversion (portnum))
		return 0;
	if (!OpenPIOS_A (portnum))
		return 0;
	strcpy (Lect, ReadPressureSensor (portnum, 16));

	if (!OpenPIOS_B (portnum))
		return 0;

	if (strlen (Lect) != 16)
		return 0;
	for (i = 0; i < 16; i++)
	{
		Res = (Res << 1);
		if (Lect[i] == '1')
			++Res;
	}
	//printf ("-> %d\n", Res);
	return Res;
}

static bool
CheckConversion (int portnum)
{
	int i;
	if (!ConfigReadPulse (portnum))
	{
		fprintf (stderr,
			 "ConfigReadPulse failed in CheckConversion\n");
		return false;
	}

	for (i = 0; i < 100; i++)
		if (!owTouchBit (portnum, 1))
			break;
	//printf ("CheckConversion returned at i=%d\n", i);
	return (i < 100);
}




/******************************************************/

static bool
ReadSensor_Calibration (int portnum)
{
	int uno = 1, im = 0, Count = 0;
	int i = 0;
	int Lect = false;
	char SW11[20] = "0", SW22[20] = "0", SW33[20] = "0", SW44[20] = "0";

	do
	{
		if (!PressureReset (portnum))
			return false;
		ReadSensorValueStr (portnum, SEC_READW1, SW1);
		ReadSensorValueStr (portnum, SEC_READW2, SW2);
		ReadSensorValueStr (portnum, SEC_READW3, SW3);
		ReadSensorValueStr (portnum, SEC_READW4, SW4);

		if (strlen (SW1) != 16)
			return false;
		if (strlen (SW2) != 16)
			return false;
		if (strlen (SW3) != 16)
			return false;
		if (strlen (SW4) != 16)
			return false;

		Lect = ((strcmp (SW1, SW11) == 0)
			&& (strcmp (SW2, SW22) == 0))
			&& ((strcmp (SW3, SW33) == 0)
			    && (strcmp (SW4, SW44) == 0));

		strcpy (SW11, SW1);
		strcpy (SW22, SW2);
		strcpy (SW33, SW3);
		strcpy (SW44, SW4);

		Count++;
	}
	while ((Count < 5) && (Lect == false));



	C1 = 0;
	C2 = 0;
	C3 = 0;
	C4 = 0;
	C5 = 0;
	C6 = 0;

	for (i = 14; i >= 0; i--)
	{
		if (SW1[i] == '1')
		{
			C1 = C1 | (uno << (14 - i));
		}
	}

	for (i = 9; i >= 0; i--)
	{
		if (SW2[i] == '1')
			C5 = C5 | (uno << (9 - i));
	}
	if (SW1[15] == '1')
		C5 = C5 | (uno << (10));

	for (i = 15; i >= 10; i--)
	{
		if (SW2[i] == '1')
			C6 = C6 | (uno << (15 - i));
	}

	for (i = 9; i >= 0; i--)
	{
		if (SW3[i] == '1')
			C4 = C4 | (uno << (9 - i));
	}

	for (i = 9; i >= 0; i--)
	{
		if (SW4[i] == '1')
			C3 = C3 | (uno << (9 - i));
	}

	im = 0;
	for (i = 15; i >= 10; i--)
	{
		if (SW4[i] == '1')
			C2 = C2 | (uno << (im));
		im++;
	}

	for (i = 15; i >= 10; i--)
	{
		if (SW3[i] == '1')
			C2 = C2 | (uno << (im));
		im++;
	}


	printf ("C1=%d\n", C1);
	printf ("C2=%d\n", C2);
	printf ("C3=%d\n", C3);
	printf ("C4=%d\n", C4);
	printf ("C5=%d\n", C5);
	printf ("C6=%d\n", C6);

	return true;
}

/******************************************************/

static bool
ReadSensor_Values (int portnum)
{
	double DT, OFF, SENSE, X;

	D1 = ReadSensorValue (portnum, SEC_READD1);
	D2 = ReadSensorValue (portnum, SEC_READD2);
	DT = D2 - ((8 * C5) + 20224.0);
	OFF = C2 * 4 + (((C4 - 512) * DT) / 4096);
	SENSE = 24576 + C1 + ((C3 * DT) / 1024);
	X = ((SENSE * (D1 - 7168)) / 16384) - OFF;
	Pressure_mbar = 250 + X / 32;
	Temp_C = 20 + ((DT * (C6 + 50)) / 10240);
	return true;
}

#if 0
    /** Detecta si el el modulo TAI8570 esta presente en la red
     * @return bool
     */
bool
isPresent ()
{
	return true;
}

void
readDevice (void)
{
	if (!validCFG)
	{
		if (!ReadSensor_Calibration ())
			throw new
				Exception
				("Error reading calibration constants");
		validCFG = true;
	}
	if (!ReadSensor_Values ())
		throw new
			Exception
			("Error reading Pressure and Temperature values");
}

double
getPressure_mbar ()
     throws Exception
     {
	     return Pressure_mbar;
     }

     String getPressure_mbar_S ()
     throws Exception
     {
	     return Double.toString (getPressure_mbar ());
     }

     double getPressure_atm ()
     throws Exception
     {
	     return (getPressure_mbar () * 9.872E-4);
     }

     String getPressure_atm_S ()
     throws Exception
     {
	     return Double.toString (getPressure_atm ());
     }

     double getPressure_KgCm2 ()
     throws Exception
     {
	     return (getPressure_mbar () * 1.0200E-3);
     }

     String getPressure_KgCm2_S ()
     throws Exception
     {
	     return Double.toString (getPressure_KgCm2 ());
     }

     double getPressure_inHg ()
     throws Exception
     {
	     return (getPressure_mbar () * 2.954E-2);
     }

     String getPressure_inHg_S ()
     throws Exception
     {
	     return Double.toString (getPressure_inHg ());
     }

     double getPressure_inH2O ()
     throws Exception
     {
	     return (getPressure_mbar () * 0.4018);
     }

     String getPressuree_inH2O_S ()
     throws Exception
     {
	     return Double.toString (getPressure_inH2O ());
     }

     double getPressure_Pa ()
     throws Exception
     {
	     return (getPressure_mbar () * 100);
     }

     String getPressure_Pa_S ()
     throws Exception
     {
	     return Double.toString (getPressure_Pa ());
     }

     double getPressure_psi ()
     throws Exception
     {
	     return (getPressure_mbar () * 1.450E-2);
     }

     String getPressure_psi_S ()
     throws Exception
     {
	     return Double.toString (getPressure_psi ());
     }


     double getPressure_Sea_mbar ()
     throws Exception
     {
	     return 0.0;
     }

     String getPressure_Sea_mbar_S ()
     throws Exception
     {
	     return Double.toString (getPressure_Sea_mbar ());
     }

     double getPressure_Sea_atm ()
     throws Exception
     {
	     return (getPressure_Sea_mbar () * 9.872E-4);
     }

     String getPressure_Sea_atm_S ()
     throws Exception
     {
	     return Double.toString (getPressure_Sea_atm ());
     }

     double getTemp_C ()
     throws Exception
     {
	     return Temp_C;
     }

     double getTemp_F ()
     throws Exception
     {
	     return (1.8 * getTemp_C () + 32);
     }

     String getTemp_C_S ()
     throws Exception
     {
	     return Double.toString (getTemp_C ());

     }

     String getTemp_F_S ()
     throws Exception
     {
	     return Double.toString (getTemp_F ());
     }

     void set_height_over_sea_level_meter (float height)
{
	height_meters = height;
}

void
set_height_over_sea_level_feet (float height)
{
	height_meters = height * 0.3048;
}

double
get_height_over_sea_level_meter ()
     throws Exception
     {
	     return height_meters;
     }

     double get_height_over_sea_level_feet ()
     throws Exception
     {
	     return (3.281 * height_meters);
     }

     double get_calc_height_over_sea_level_meter ()
     throws Exception
     {
	     double Result =
		     (288.15 / 0.0065) * (1 -
					  pow ((double)
					       (getPressure_Pa () / 101325),
					       (double) (0.0065 *
							 (287.052 / 9.81))));
     }
#endif

//--------------------------------------------------------------------------
// This is the begining of the program that tests the sensor
     int main (int argc, char **argv)
{
	int i, k;		//loop counters
	int n;

  time_t t0, t1 ;
	//short test=0;                   //info byte data
	//short clear=0;                  //used to clear the button
	//SwitchProps sw;                 //used to set Channel A and B
	//uchar SwitchSN[MAXDEVICES][8];  //the serial numbers for the devices
	int num;		//for the number of devices present
	//char out[140];                  //used for output of the info byte data
	int portnum = 0;

	//----------------------------------------
	// Introduction header
	printf ("\n/---------------------------------------------\n");
	printf ("  tai8570 - V1.01\n"
		"  The following is a test to exercise the TAI8570\n");
	printf ("  Press CTRL-C to stop this program.\n\n");

	// check for required port name
	if (argc != 2)
	{
		printf ("1-Wire Net name required on command line!\n"
			" (example: \"COM1\" (Win32 DS2480),\"/dev/cua0\" "
			"(Linux DS2480),\"1\" (Win32 TMEX)\n");
		exit (1);
	}

	// attempt to acquire the 1-Wire Net
	if (!owAcquire (portnum, argv[1]))
	{
		OWERROR_DUMP (stdout);
		exit (1);
	}

	// success
	printf ("Port opened: %s\n", argv[1]);

	// this is to get the number of the devices and the serial numbers
	num = FindDevices (portnum, &SwitchSN[0], SWITCH_FAMILY, MAXDEVICES);

	// setting up the first print out for the frist device
	owSerialNum (portnum, SwitchSN[0], FALSE);

	n = 0;
	reader = -1 ;

	printf ("\n\n");
	for (k = 0; k < num; k++)
	{
		int found = 0, fl_len ;
		short hnd ;
		FileEntry fe ;
		uchar buf[100] ;

		printf ("%d  ", k);
		for (i = 7; i >= 0; i--)
		{
			printf ("%02X", SwitchSN[k][i]);
		}
		printf ("\n");

		memcpy(fe.Name, "8570", 4) ;
		fe.Ext = '\0' ;

		found = owOpenFile(portnum, SwitchSN[k], &fe, &hnd) ;

		if (found)
		{
		  int j, info, isreader = 0 ;
		  owReadFile(portnum, SwitchSN[k], hnd, buf, 99, &fl_len) ;
      
		  owSerialNum (portnum, SwitchSN[k], false);
      info = ReadSwitch12(portnum, false) ;
      if (info == -1)
      {
        fprintf(stderr, "Unable to read info byte\n") ;
        exit(1) ;
      }
      
      if ((info & 0x80) == 0) isreader = true ;

      if (isreader)
		    printf("Found 8570 reader. Writer is:\n   ") ;
      else
        printf("Found 8570 writer. Reader is:\n   ") ;

		  for (j=7; j>=0; --j)
		  {
		    printf("%02X", buf[j]) ;
		    SwitchSN[k+1][j] = buf[j] ;
		  }

      if (isreader)
      {
	  	  reader = k ;
		    writer = k+1 ;
      }
      else
      {
	  	  reader = k+1 ;
		    writer = k ;
      }

		  printf("\nProceeding automatically\n") ;

		  break ;
		}
	}

	if (reader == -1)
	{
		printf ("%d To quit.\n", k);

		printf ("\n");
		printf ("Pick a reader device\n");

		n = getNumber (0, num);

		if (n == num)
		exit(0) ;

		reader = n;

		printf ("\n");
		printf ("Pick a writer device\n");

		n = getNumber (0, num);

		if (n == num)
		exit(0) ;

		writer = n;
	}

	/*printf ("\nReader: %d  ", reader);
	for (i = 7; i >= 0; i--)
	{
		printf ("%02X", SwitchSN[reader][i]);
	}
	printf ("\nWriter: %d  ", writer);
	for (i = 7; i >= 0; i--)
	{
		printf ("%02X", SwitchSN[writer][i]);
	}*/

	if (!validCFG)
	{
    time(&t0) ;
    printf("\nReading calibration constants. Please wait...\n") ;
		if (!ReadSensor_Calibration (portnum))
		{
			fprintf (stderr,
				 "Error reading calibration constants\n");
			exit (1);
		}
    time(&t1) ;
    printf("Time to read: %d s\n", t1-t0) ;
		validCFG = true;
	}
  
	while (1)
	{
    t0 = t1 ;
		if (!ReadSensor_Values (portnum))
		{
			fprintf (stderr,
				 "Error reading Pressure and Temperature values");
			exit (1);
		}

		printf ("Temperature:\t%f\n", Temp_C);
		printf ("Presuure:   \t%f\n", Pressure_mbar);
    
    time(&t1) ;
    printf("Time to read values: %d s\n\n", t1-t0) ;
 	}
	

	/*owRelease (portnum);
	printf ("Closing port %s.\n", argv[1]);*/
	exit (0);

	return 0;
}

/**
 * Retrieve user input from the console.
 *
 * min  minimum number to accept
 * max  maximum number to accept
 *
 * @return numeric value entered from the console.
 */
int
getNumber (int min, int max)
{
	int value = min, cnt;
	int done = FALSE;
	do
	{
		cnt = scanf ("%d", &value);
		if (cnt > 0 && (value > max || value < min))
		{
			printf ("Value (%d) is outside of the limits (%d,%d)\n", value, min, max);
			printf ("Try again:\n");
		}
		else
			done = TRUE;

	}
	while (!done);

	return value;

}
